-- |Data Dictionary module.
-- The data dictionary allows widgets and controls
-- to be collected in a single place.
module DataDict where

import Data.Maybe

-- Some types for widget handling

-- |Available buttons
data Button = OptButton
             | ResetButton deriving (Eq,Show)
-- |Available texts
data Text = EAPText
           | FAPText
           | RatioText
           | FieldText deriving (Eq,Show)
-- |Available controls
data Control = GainSlider deriving (Eq,Show)

-- |A dictionary key
data DictKey = ButtonKey Button
             | TextKey Text
             | ControlKey Control deriving (Eq,Show)

-- |An individual dictionary. Multiple dictionaries are
-- collected into the data dictionary.
data Dict a = Dict [(DictKey,a)] deriving (Show)

-- |The data dictionary itself is a collection of buttons, text,
-- and GUI controls.
data DataDict a b c =
    DataDict { buttonDict :: Dict a,
               textDict :: Dict b,
               controlDict :: Dict c }

emptyDict = Dict []

-- |Look up an entry in an individual dictionary
lookupEntry :: DictKey -> Dict a -> Maybe a
lookupEntry key (Dict d) =
    lookup key d

-- |Get an entry from an individual dictionary. If the entry
-- doesn't exist, this will throw an exception.
getEntry :: DictKey -> Dict a -> a
getEntry key d =
    fromJust (lookupEntry key d)

-- |Add an entry to an individual dictionary. If the entry
-- already exists, nothing happens
addEntry :: DictKey -> a -> Dict a -> Dict a
addEntry key val (Dict d) =
    case (lookup key d) of
        Nothing -> Dict ((key,val):d)
        Just v -> Dict d


-- Dictionary specific gets

-- |Get a button from the data dictionary
getButton :: Button -> DataDict a b c -> a
getButton k dd = getEntry (ButtonKey k) (buttonDict dd)

-- |Get a text from the data dictionary
getText :: Text -> DataDict a b c -> b
getText k dd = getEntry (TextKey k) (textDict dd)

-- |Get a control from the data dictionary
getControl :: Control -> DataDict a b c -> c
getControl k dd = getEntry (ControlKey k) (controlDict dd)



